const { convertRequest } = require('./helpers')
const { timeoutPromise } = require('./promiseHelpers')
const generateId = require('shortid')

exports.createRequest = function (peer, rawRequest, timeoutLength = 600000) {
  // Convert raw request
  const convertedRequest = convertRequest(rawRequest)

  const timeout = timeoutPromise(timeoutLength)

  return {
    ...convertedRequest,
    id: generateId(),
    timeout,
    respond () {
      // Cancel the timeout
      timeout.cancel()
      return peer
        .call(this.callbackPath, {
          resources: this.requestedResources.map(
            requestedResource => requestedResource.resource
          )
        })
        .catch(err => {
          console.log(
            `Error responding to resource request by instructor ${this.from}:`,
            err
          )
        })
    },
    cancel () {
      return peer.call(this.callbackPath, {
        error: `Error responding to resource request: Timeout of ${timeoutLength}ms has been reached.`
      })
    }
  }
}
