const generateId = require('shortid')

exports.createPartnerManager = function (peer, onResourceProvided) {
  return {
    async add (db, resourceData) {
      await db.set('partners', resourceData)
      return onResourceProvided(db)
    },
    release (db, id) {
      return peer.call(`/partners/${id}/methods/wo-deregister`).catch(() => {
        console.warn(`Partner with id "${id}" already deregistered.`)
      })
    }
  }
}

exports.createUserManager = function (onResourceProvided) {
  return {
    add (db, resourceData) {
      const id = generateId()
      return db.add('users', {
        id,
        value: resourceData.value
      })
    },
    async release (db, id) {
      const query = {
        path: {
          equals: `/users/${id}`
        }
      }
      const [user] = await db.get(query)

      if (!user) {
        console.error(
          'Failed to find a user with query while trying to release it. Query: ',
          query
        )

        const allUsers = await db.get({
          path: {
            startsWith: '/users/'
          }
        })
        console.debug('All users: ', allUsers)

        return onResourceProvided(db)
      }

      try {
        await db.set('users', {
          id,
          value: {
            ...user.value,
            reservedBy: null,
            isAvailable: true
          }
        })
      } catch (e) {
        console.error('Failed to release user: ', e)
      }

      return onResourceProvided(db)
    }
  }
}
